;;; widget/tag-list.fnl --- Pop-out tag list widget

;;; Code:

(local awful (require :awful))
(local beautiful (require :beautiful))
(local wibox (require :wibox))
(local common (require :awful.widget.common))
(local dpi (. (require :beautiful) :xresources :apply_dpi))

(local std (require :lib.std))

;;;
;; Functions

(fn update-tag-list [w buttons label data objects]
  (: w :reset)
  (each [i o (ipairs objects)]
    (let [cache (. data o)]
      (var ib nil)
      (var tb nil)
      (var bgb nil)
      (var tbm nil)
      (var ibm nil)
      (var l nil)
      (if cache
          (do
            (set ib cache.ib)
            (set tb cache.tb)
            (set bgb cache.bgb)
            (set tbm cache.tbm)
            (set ibm cache.ibm))
          (do
            (set ib (wibox.widget.imagebox))
            (set tb (wibox.widget.textbox))
            (set bgb (wibox.container.background))
            (set tbm (wibox.container.margin tb (dpi 4) (dpi 4)))
            ;; (set ibm (wibox.container.margin ib (dpi 12) (dpi 12) (dpi 12) (dpi 12)))
            (set ibm (wibox.container.margin ib (dpi 4)))
            (set l (wibox.layout.fixed.horizontal))

            ;; All of this is added in a fixed widget
            (doto l
              (: :fill_space true)
              (: :add ibm)
              (: :add tbm))

            ;; And all of this gets a background
            (: bgb :set_widget l)

            (: bgb :buttons (common.create_buttons buttons o))

            (tset data o {:ib ib :tb tb :bgb bgb :tbm tbm :ibm ibm})))

      (let [[text bg bg-image icon args] (label o tb)]
        ;; The text might be invalid, so use pcall.
        (if (std.s-empty? text)
            (: tbm :set_margins 0)
            (when (not (: tb :set_markup_silently text))
              (: tb :set_markup "<i>&lt;Invalid text&gt;</i>")))
        (: bgb :set_bg bg)

        (: bgb :set_bgimage (if (std.fn? bg-image)
                                ;; TODO: Why does this pass nil as an argument?
                                (bg-image tb o nil objects i)
                                bg-image))

        (if icon
            (: ib :set_image icon)
            (: ibm :set_margins 0))

        (let [args (or args {})]
         (doto bgb
           (tset :shape args.shape)
           (tset :shape_border_width args.shape_border_width)
           (tset :shape_border_color args.shape_border_color)))

        (: w :add bgb)))))

(fn container []
  (let [l (wibox.layout.fixed.vertical)]
    ;; Wishlist: let*
    (local innerm (wibox.container.margin nil 10 10))
    (local bg (wibox.container.background innerm "#faa"))
    (local outerm (wibox.container.margin bg 4 4))

    (: l :fill_space true)
    (: l :add outerm)

    l))

(fn tag-list [s]
  (awful.widget.taglist
   s
   awful.widget.taglist.filter.all
   {}
   {:font (.. beautiful.font " Bold 10")
    :spacing 2}
   ;; update-tag-list
   (require :widget.list_update)
   ;; awful.widget.common.list_update
   ;; (container)
   (wibox.layout.fixed.vertical)
   ))

tag-list
;;; widget/tag-list.fnl ends here
